# Corpus - A class representing a collection of documents.

setClass("Corpus",
    representation(
        id          =   "character",
        documents   =   "list"
    )
)


"Corpus" <-
# Constructor for corpus objects
function(id="corpus", documents=list()) {
    c <- new("Corpus", id=id, documents=documents)
    return(c)
}


"isValidDocumentList" <-
# A utility function that determines whether the given list contains
# only documents, that the list keys are equal to the document ids,
# and that each document has a unique id. Calls "warning" if the list does
# not meet these requirements and returns FALSE.
function(lst) {
    listKeys <- names(lst)
    for (key in listKeys) {
        object <- lst[[key]]
        if (is(object, "Document")) {
            objectId <- id(object)
            if (objectId != key) {
                msg <- paste("list key does not match id for document", 
                    objectId)
                warning(msg)
                return(FALSE)
            }
        } else {
            msg <- paste("object", object, "is not a document")
            warning(msg)
            return(FALSE)
        }
    }
    return(TRUE)
}


"setCorpusDocuments" <-
# Sets the documents of this corpus to the given list. Must be a list
# of only document objects with the list keys equal to the document ids.
function(object, value) {
    if (isValidDocumentList(value)) {
        object@documents <- value
        return(object)
    } else {
        stop("invalid document list")
    }
}
setMethod("documents<-", signature(object="Corpus", value="list"),
    setCorpusDocuments)



"getCorpusDocuments" <-
# Returns the documents of this corpus as a list indexed by document id.
function(object) {
    return(object@documents)
}
setMethod("documents", "Corpus", getCorpusDocuments)


"getCorpusId" <-
# Returns the id of this corpus
function(object) {
    return(object@id)
}
setMethod("id", "Corpus", getCorpusId)


"setCorpusId" <-
# Sets the id of this corpus
function(object, value) {
    if (length(value) == 1 && nchar(value) > 0) {
        object@id <- value
        return(object)
    } else {
        stop("id must have length 1 and nchar > 0")
    }
}
setMethod("id<-", signature(object="Corpus", value="character"), setCorpusId)


"corpusHasDocumentId" <-
# Returns TRUE if corpus has a document with the given id, FALSE otherwise.
function(object, doc) {
    ids <- names(documents(object))
    if (doc %in% ids) {
        return(TRUE)
    } else {
        return(FALSE)
    }
}
setMethod("hasDocument", signature(object="Corpus", doc="character"),
    corpusHasDocumentId)


"corpusHasDocumentDoc" <-
# Returns TRUE if corpus contains a document with the same id as the given 
# document, FALSE otherwise. 
function(object, doc) {
    docId <- id(doc)
    return(hasDocument(object, docId)) 
                                       
}
setMethod("hasDocument", signature(object="Corpus", doc="Document"),
    corpusHasDocumentDoc)


"selectCorpusDocument" <-
# Returns the document object with the given id
function(x, i, j, drop) {

    if (!missing(j)) {
        warning("ignoring second subscript")
    }

    if (length(i) != 1) {
        stop("can only select one document at a time")
    }

    if (!hasDocument(x, i)) {
        stop(paste("corpus does not contain document", i))
    }

    docs <- documents(x)
    return(docs[[i]])

}
setMethod("[", signature(x="Corpus", i="character"), selectCorpusDocument)


"replaceCorpusDocument" <-
# Adds the given document to the corpus and sets the id to i. If the original
# id of the document does not match i, a warning will be given.
function(x, i, j, value) {

    if (length(i) != 1) {
        stop("index must have length 1")
    }

    if (!missing(j)) {
        warning("ignoring second subscript")
    }

    docId <- id(value)
    if (docId != i) {
        warning(paste("changing id of document", docId))
        id(value) <- i
    }

    docs <- documents(x)
    docs[[i]] <- value
    documents(x) <- docs
    return(x)
   
}
setMethod("[<-", signature(x="Corpus", i="character", value="Document"),
    replaceCorpusDocument)


"isValidCorpus" <-
# A valid corpus contains a list of only document objects, with the list 
# key of a document equal to the id of that document. The document
# ids are unique identifiers in a corpus object, so there can be no 
# duplicates. The corpus must have a single character id with nchar(id) > 0.
function(object) {
    if (!isValidDocumentList(documents(object))) {
        return("invalid document list")
    }
    objectId <- id(object)
    if (length(objectId) != 1 || nchar(objectId) < 1) {
        return("id must have length 1 and nchar > 0")
    }
    return(TRUE)
}
setValidity("Corpus", isValidCorpus)


"corpusLength" <-
function(x) {
    return(length(documents(x)))
}
setMethod("length", "Corpus", corpusLength)
