# All versions of the tag method work by breaking down the given
# data object type into a vector of tokens of type character.  The
# baseline definitions dealing with character objects must all
# distinguish between character vectors of length 1, and all other
# vectors.  Longer vectors are assumed to contain a series of tokens,
# while 1-vectors are passed to a tokenizer before processing because
# they are assumed to contain simple text strings.

# A helper function for dealing with character objects
tag.character.helper <- function(object) {
  tokens <- NULL
  if (length(object) == 1)
    tokens <- tokenize(object)
  else
    tokens <- object
  return(tokens)
}

# Makes DefaultTagger the default tagger
setMethod("tag", signature(tagger="missing"),
    function(object, tagger) {
        tagger <- DefaultTagger()
        return(tag(object, tagger))
    }
)

#### Handling various taggers below here

# Tag a list of character strings with a DefaultTagger
setMethod("tag", signature(object="character", 
                           tagger="DefaultTagger"),
  function(object, tagger) {
    tokens <- tag.character.helper(object)
    res <- cbind(tokens=tokens, tags=defaultTag(tagger))
    return(res)
  }
)

# Takes care of all calls to taggers that execute through the tag
# function in the C interface.
setMethod("tag", signature(object="character", tagger="WrappedTagger"),
  function(object, tagger) {
    tokens <- tag.character.helper(object)
    return(.Call("tag", tagger@pointer, tokens))
  }
)

#### Handling various data types below here
    
# Tag a tokenized documentview with the given tagger
setMethod("tag", signature(object="TokenizedDocumentView",
                           tagger="Tagger"),
  function(object, tagger) {
    toks <- tokens(object)
    tags <- tag(toks, tagger)[,2]
    return(TaggedDocumentView(document(object), toks, tags))
  }
)

# Tag a document with the given tagger (and default tokenizer)
setMethod("tag", signature(object="Document",
                           tagger="Tagger"),
  function(object, tagger) {
    return(tag(tokenize(object), tagger))
  }
)

# Tag all the documents in a tokenized corpus view
setMethod("tag", signature(object="TokenizedCorpusView",
                           tagger="Tagger"),
  function(object, tagger) {
    tagViews <- documentViews(object)
    for (i in 1:length(tagViews)) {
      tagViews[[i]] <- tag(tagViews[[i]], tagger)
    }
    return(TaggedCorpusView(corpus(object), tagViews))
  }
)

# Tag all the documents in a corpus (after tokenizing default)
setMethod("tag", signature(object="Corpus", tagger="Tagger"),
  function(object, tagger) {
    return(tag(tokenize(object), tagger))
  }
)
