# TaggedDocumentView - A view of a document as a sequence of token-tag
# pairs
setClass("TaggedDocumentView",
    representation(
        "TokenizedDocumentView",
        tags              = "character",
        tagFreqTable      = "table",
        uniqueTags        = "character",
        tokenTagFreqTable = "table",
        uniqueTokenTags   = "character"
    )
)

"TaggedDocumentView" <-
# Constructs a new TaggedDocumentView
function(document, tokens, tags, tokenFreqTable=NULL, 
         tagFreqTable=NULL, tokenTagFreqTable=NULL)
{
  if (is.null(tokenFreqTable))
    tokenFreqTable <- table(tokens)
  if (is.null(tagFreqTable))
    tagFreqTable <- table(tags)
  if (is.null(tokenTagFreqTable))
    tokenTagFreqTable <- table(paste(tokens, tags, sep="/"))

  uniqueTokens <- row.names(tokenFreqTable)
  uniqueTags <- row.names(tagFreqTable)
  uniqueTokenTags <- row.names(tokenTagFreqTable)

  new("TaggedDocumentView",
    document=document, tokens=tokens, tags=tags,
    tokenFreqTable=tokenFreqTable, tagFreqTable=tagFreqTable,
    tokenTagFreqTable=tokenTagFreqTable, uniqueTokens=uniqueTokens,
    uniqueTags=uniqueTags, uniqueTokenTags=uniqueTokenTags)
}

"getTaggedDocumentViewTags" <-
# Retunrs the tags of this TaggedDocumentView
function(object) {
  return(object@tags)
}

setMethod("tags", "TaggedDocumentView", getTaggedDocumentViewTags)

"getTaggedDocumentViewTaggedTokens" <-
# Returns token-tag pairs of this TaggedDocumentView
function(object) {
  paste(object@tokens, object@tags, sep="/")
}

setMethod("taggedTokens", "TaggedDocumentView",
  getTaggedDocumentViewTaggedTokens)

"taggedDocumentViewExtract" <-
# Return tokens/tags (or pairs) at given indices.  x[i, 1] returns the
# token at index i, x[i, 2] returns the tag at index i, and x[i]
# returns the token-tag pair at index i.
function(x, i, j, drop) {
  if (missing(j)) {
    return(c(tokens(x)[i], tags(x)[i]))
  } else if (j == 1) {
    return(tokens(x)[i])
  } else if (j == 2) {
    return(tags(x)[i])
  } else {
    warning("Second argument to extractor must be 1 (token) or 2 (tag)")
  }
  return(NA)
}
 
 setMethod("[", signature(x="TaggedDocumentView"),
  taggedDocumentViewExtract)

"getTaggedDocumentViewFreqTable" <-
# Returns the frequency table of tagged tokens
function(object, ...) {
  return(object@tokenTagFreqTable)
}

setMethod("freqTable", "TaggedDocumentView", 
  getTaggedDocumentViewFreqTable)

# this func in tokenizedDocumentView-class.R
setMethod("tokenFreqTable", "TaggedDocumentView", 
  getTokenizedDocumentViewFreqTable)

"getTaggedDocumentViewTagFreqTable" <-
# Returns the frequency table of tags
function(object, ...) {
  return(object@tagFreqTable)
}

setMethod("tagFreqTable", "TaggedDocumentView",
  getTaggedDocumentViewTagFreqTable)

"getTaggedDocumentViewUniqueTaggedTokens" <-
# Returns the unique token-tags in the document
function(x, incomparables="missing") {
  return(x@uniqueTokenTags)
}

setMethod("unique", "TaggedDocumentView",
  getTaggedDocumentViewUniqueTaggedTokens)

# this func in tokenizedDocumentView-class.R
setMethod("uniqueTokens", "TaggedDocumentView",
  getTokenizedDocumentViewUniqueTokens)

"getTaggedDocumentViewUniqueTags" <-
# Returns the unique tags in the document
function(object) {
  return(object@uniqueTags)
}

setMethod("uniqueTags", "TaggedDocumentView",
  getTaggedDocumentViewUniqueTags)


# NOTE: inherits has method from TokenizedDocumentView.  You pass a
# string of the form token/tag to the method.  Also, can explicitly
# call hasTaggedToken with different syntax.

"taggedDocumentViewHasTaggedToken" <-
# Retunrs TRUE if this contains given token with given tag
function(token, tag, container, ...) {
  return(has(paste(token, tag, sep="/"), container, ...))
}

setMethod("hasTaggedToken",
  signature(token="character", tag="character", 
            container="TaggedDocumentView"), 
  taggedDocumentViewHasTaggedToken)

"taggedDocumentViewHasToken" <-
# Returns TRUE if this contains the given token
function(object, container, ...) {
  return(object %in% uniqueTokens(container))
}

setMethod("hasToken", 
  signature(object="character", container="TaggedDocumentView"),
  taggedDocumentViewHasToken)

"taggedDocumentViewHasTag" <-
# Returns TRUE if this contains given tag
function(object, container, ...) {
  return(object %in% uniqueTags(container))
}

setMethod("hasTag", 
  signature(object="character", container="TaggedDocumentView"),
  taggedDocumentViewHasTag)

