"regexTokenizeMatchTokens" <- 
# A utility function for tokenizing a document based on tokens instead of
# based on separators
function(str, pattern) {

    # Find substrings that match the pattern
    res <- gregexpr(pattern, str, perl=TRUE)

    # Get the start indices
    startInds <- res[[1]]

    # the case of no matches
    if (-1 %in% startInds) {
        return(character(0))
    }

    # The lengths of each match are stored in the match.length attrbute
    # and the end indices of each match can be calculated from them
    endInds <- startInds + attr(startInds, "match.length") - 1
    nTokens <- length(startInds)

    # Store the tokens in a character vector and return them
    tokens <- character(nTokens)
    for (i in 1:nTokens) {
        tokens[i] <- substr(str, startInds[i], endInds[i])
    }

    return(tokens)

}


"regexTokenizeMatchSeps" <-
# A utility function for tokenizing a document based on separators.
function(str, pattern) {
    return(unlist(strsplit(str, pattern, perl=TRUE)))
}


"regexTokenizeChar" <-
# Tokenizes the given string with the given regexTokenizer object. Returns
# a character vector containing tokens.
function(object, tokenizer) {
    if (length(object) != 1) {
        stop("object must have length 1")
    }
    res <- NULL
    if (matchToken(tokenizer)) {
        res <- regexTokenizeMatchTokens(object, pattern(tokenizer))
    } else {
        res <- regexTokenizeMatchSeps(object, pattern(tokenizer))
    }
    return(res[ res != "" ])
}
setMethod("tokenize", signature(object="character", 
  tokenizer="RegexTokenizer"), regexTokenizeChar)

# Setup tokenizing for PunktWordTokenizers
setMethod("tokenize", signature(object="character",
                                tokenizer="PunktWordTokenizer"), 
  function(object, tokenizer) {
    if (length(object) != 1) {
        stop("object must have length 1")
    }
    return(.Call("tokenize", tokenizer@pointer, object));
  }
)

"offsetTokenizeChar" <-
# Tokenizes the given string with an OffsetTokenizer, returning a
# character vector.
function(object, tokenizer) {
  if (length(object) != 1) {
    stop("object must have length 1")
  }

  offsets <- offsets(tokenizer)
  if (offsets[length(offsets)] > nchar(object))
    stop("Offsets cannot extend past end of string")

  offsets <- cbind(offsets, c(offsets[-1] - 1, nchar(object)))
  apply(offsets, 1, function(offset) {
    substr(object, offset[1], offset[2])})
}
setMethod("tokenize", signature(object="character",
          tokenizer="OffsetTokenizer"), offsetTokenizeChar)

# Makes regexTokenizer the default tokenizer
setMethod("tokenize", signature(tokenizer="missing"),
    function(object, tokenizer) {
        tokenizer <- RegexTokenizer()
        return(tokenize(object, tokenizer))
    }
)


"tokenizeDocument" <-
# Tokenizes the given document and returns a tokenizedDocumentView
function(object, tokenizer) {
    str <- content(object)
    toks <- tokenize(str, tokenizer)

    view <- TokenizedDocumentView(document=object, tokens=toks)
        
    return(view)
}
setMethod("tokenize", signature(object="Document",
  tokenizer="Tokenizer"), tokenizeDocument)


"tokenizeCorpus" <-
# Tokenizes the documents in the given corpus and then returns a new
# tokenizedCorpusView.
function(object, tokenizer) {
    tdViews <- list()
    docs <- documents(object)
    for (doc in docs) {
        tdView <- tokenize(doc, tokenizer)
        docId <- id(doc)
        tdViews[[docId]] <- tdView
    }
    return(TokenizedCorpusView(object, tdViews))
}
setMethod("tokenize", signature(object="Corpus", 
  tokenizer="Tokenizer"), tokenizeCorpus)

