#ifndef __EMBED_NLTK__
#define __EMBED_NLTK__

/* This file contains basic routines for starting up and shutting down
 * the embedded python interpreter and importing the needed nltk
 * modules, and also a number of utility functions for converting
 * between R and python objects.
 *
 * Only init_python and finalize_python methods are designed to be
 * called directly from R.
 */

#include <Python.h>
#include <Rinternals.h>

/* Globals - only accessible from this file */
static PyObject* NLTK_TAG;           /* Pointer to nltk tag module */
static PyObject* NLTK_PUNKT;         /* Pointer to nltk punkt module */

/* An imp-indep way to get a pointer to the nltk.tag module */
inline PyObject*
nltk_tag_pnt ()
{
  return NLTK_TAG;
}

/* An imp-indep way to get pointer to the nltk.tokenize.punkt module */
inline PyObject*
nltk_punkt_pnt ()
{
  return NLTK_PUNKT;
}

/* Start up the embedded python interpreter and import the necessary
 * nltk modules.  
 *
 * Designed to be called from R with .Call.  Returns nil on exit.
 */
inline SEXP
init_python ()
{
  Py_Initialize();
  NLTK_TAG = PyImport_ImportModule("nltk.tag");
  NLTK_PUNKT = PyImport_ImportModule("nltk.tokenize.punkt");

  return R_NilValue;
}

/* Decref imported modules and close down the embedded interpreter.
 *
 * Designed to be called from R with .Call.  Returns nil on exit.
 */
inline SEXP
finalize_python ()
{
  Py_DECREF(nltk_tag_pnt());
  Py_DECREF(nltk_punkt_pnt());
  Py_Finalize();

  return R_NilValue;
}

/* Constructs a Python object and returns an R external pointer to it.
 * Takes a python module to find the object constructor in and a tuple
 * containing arguments to the constructor, the name of the
 * constructor as a string, and a destructor function to call when the
 * python object is no longer referenced by any R objects.  Returns an
 * SEXP pointing to an extenalptr object (which points to the python
 * object).
 *
 * !!! Note that this function steals the argtuple reference (that is,
 * it decrefs it before returning) but not the module reference, which
 * it borrows.  Also, the caller is responsible for unprotecting the
 * returned pointer before returning to R.
 */
SEXP
construct_python_object (PyObject* module, PyObject* argtuple, 
                         const char* name, R_CFinalizer_t destructor)
{
  PyObject *constructor, *object;
  SEXP ret;
  
  /* Grab the constructor */
  constructor = PyObject_GetAttrString(module, name);
  if (constructor == NULL)  {
    PyErr_Print();
    Rprintf("%s: ", name);
    error("Error fetching constructor");
  }

  /* Do the construction */
  object = PyObject_CallObject(constructor, argtuple);
  if (object == NULL) {
    PyErr_Print();
    Rprintf("%s: ", name);
    error("Error constructing");
  }

  /* Clean up a bit */
  Py_DECREF(argtuple); /* dispose of used argument */
  Py_DECREF(constructor); /* dispose of the constructor ref */

  /* Create the external pointer and register the destructor */
  PROTECT(ret = R_MakeExternalPtr(object, R_NilValue, R_NilValue));
  R_RegisterCFinalizerEx(ret, destructor, TRUE);
  
  return ret;
}

/* Clean up a python object, given its R external pointer */
inline void
destruct_python_object (SEXP externalptr)
{
  PyObject* object = (PyObject*) R_ExternalPtrAddr(externalptr);
  Py_DECREF(object);
}


/**** Utility functions (mostly R-c-python data type converters) ****/

/* A generalization of the CHAR macro in Rdefines.h.  This macro
 * returns a pointer to an array of pointers to character arrays,
 * where each pointer points to a string in the character vector x.
 *
 * Basically, you can get a pointer to the underlying c string in a
 * CHARSXP but CHAR, unlike integer, doesn't return an array of
 * pointers for a STRSXP.  We want this to avoid copying a bunch of
 * strings, although we still need to allocate a pointer array.
 *
 * !!! The one tricky thing is that you need to free your array.
 *
 * XXX Leaving CHARVEC code in for now just in case, but not looking
 * like we're going to need it.
 */
#define CHARVEC(x) point_to_chars(x)

inline const char**
point_to_chars(SEXP stringvec)
{
  int size = LENGTH(stringvec);

  const char** array =
    (const char**) malloc (sizeof(char**[size]));

  for (int i = 0; i < size; ++i)
    array[i] = CHAR(STRING_ELT(stringvec, i));

  return array;
}

/* Takes an R string vector of tokens and returns a pointer to a
 * python list containing the same tokens.
 *
 * The caller is responsible for Py_DECREFinf the returned object.
 */
inline PyObject*
r2p_token_list(SEXP r_list)
{
  PyObject *list;
  int length = LENGTH(r_list);
  
  list = PyList_New(length);
  for (int i = 0; i < length; ++i)
    PyList_SetItem(list, i, 
        PyString_FromString(CHAR(STRING_ELT(r_list, i))));

  return list;
}

/* Takes a python list of tokens and returns an R string vector
 * containing the same tokens.
 *
 * The caller is responsible for UNPROTECTing the returned object.
 */
inline SEXP
p2r_token_list(PyObject *p_list)
{
  SEXP list;
  int length = PyList_Size(p_list);
  
  PROTECT(list = allocVector(STRSXP, length));
  for (int i = 0; i < length; ++i) {
    SET_STRING_ELT(list, i, 
        mkChar(PyString_AsString(PyList_GetItem(p_list, i))));
  }

  return list;
}

/* Takes an R list representing a tagged string---that is a list of
 * lists where each sublist is a pair containing two string, a token
 * and a tag---and returns a pointer to the equivalent python object
 * (which is a list of tuple pairs).
 *
 * The caller is responsible for Py_DECREFinf the returned object.
 */
inline PyObject*
r2p_tagged_token_list (SEXP r_list)
{
  SEXP r_tokentag;
  PyObject *list, *tokentag;
  int size;
 
  size = LENGTH(r_list);
  list = PyList_New(size);
  for (int i = 0; i < size; ++i) {
    r_tokentag = VECTOR_ELT(r_list, i);
    tokentag = PyTuple_New(2);
    PyTuple_SetItem(tokentag, 0, 
      PyString_FromString(CHAR(STRING_ELT(VECTOR_ELT(r_tokentag, 0), 
            0))));
    PyTuple_SetItem(tokentag, 1, 
      PyString_FromString(CHAR(STRING_ELT(VECTOR_ELT(r_tokentag, 1), 
            0))));
    PyList_SetItem(list, i, tokentag);
  }

  return list;
}

/* A helper that converts the python object to the python string "" if
 * if it python's None, and just lets the string pass through
 * otherwise.
 */

inline PyObject*
convert_none(PyObject* string)
{
  if (PyString_Check(string))
    return string;
  else if (string == Py_None)
    return PyString_FromString("");
  else
    error("Attempted de-none-ify non-string");
}

/* Takes a python list of tagged tokens (each in a tuple pair) and
 * returns an R list
 *
 * The caller is responsible for UNPROTECTing the returned object.
 */
inline SEXP
p2r_tagged_token_list(PyObject *p_list)
{
  PyObject *p_tokentag;
  SEXP list, tokentag;
  int length = PyList_Size(p_list);
  
  PROTECT(list = allocVector(VECSXP, length));
  for (int i = 0; i < length; ++i) {
    p_tokentag = PyList_GetItem(p_list, i);
    PROTECT(tokentag = allocVector(STRSXP, 2));
    SET_STRING_ELT(tokentag, 0, 
        mkChar(PyString_AsString(convert_none(PyTuple_GetItem(p_tokentag, 0)))));
    SET_STRING_ELT(tokentag, 1, 
        mkChar(PyString_AsString(convert_none(PyTuple_GetItem(p_tokentag, 1)))));
    SET_VECTOR_ELT(list, i, tokentag);
    UNPROTECT(1); /* tokentag is now protected by list */
  }

  return list;
}

/* Takes a python list of tagged tokens (each in a tuple pair) and
 * returns a 2-columnt R matrix.
 *
 * The caller is responsible for UNPROTECTing the returned object.
 */
inline SEXP
p2r_tagged_token_list2matrix (PyObject* p_list)
{
  PyObject *p_tokentag;
  SEXP matrix, tokentag;
  int length = PyList_Size(p_list);
  
  PROTECT(matrix = allocMatrix(STRSXP, length, 2));
  for (int i = 0; i < length; ++i) {
    p_tokentag = PyList_GetItem(p_list, i);
    SET_STRING_ELT(matrix, i, 
        mkChar(PyString_AsString(convert_none(PyTuple_GetItem(p_tokentag, 0)))));
    SET_STRING_ELT(matrix, i + length, 
        mkChar(PyString_AsString(convert_none(PyTuple_GetItem(p_tokentag, 1)))));
  }

  return matrix;
}

/* Takes an R list representing a set of tagged sentences---that is, a
 * list of lists where each sublist represents a sentence and contains
 * a series of lists (pairs) each containing a string tag and a
 * token---and returns a pointer to the equivalent python object
 * (which is a list of lists of tuple pairs).
 *
 * The caller is responsible for Py_DECREFinf the returned object.
 */
inline PyObject*
r2p_tagged_token_sentences (SEXP r_list)
{
  PyObject *list;
  int nsent = LENGTH(r_list);

  list = PyList_New(nsent);
  for (int i = 0; i < nsent; ++i)
    PyList_SetItem(list, i, 
        r2p_tagged_token_list(VECTOR_ELT(r_list, i)));

  return list;
}

#endif /* __EMBED_NLTK__ */
