#ifndef __TAGGER_H__
#define __TAGGER_H__

/* Contains functions that work on all taggers.  Basically these are
 * wrappers to methods shared by all nltk taggers, most notably tag.
 *
 * Also contains some tagger-specific utility functions.
 */

#include <Python.h>
#include <Rinternals.h>
#include <Rdefines.h>
#include "embed_nltk.h"

/* Tags a chunk of text using any tagger.  Takes an external pointer
 * to a tagger and an R string vector containing tokens and
 * returns a 2-column R character matrix, with tokens in the first
 * column and tags in the second.
 *
 * Called directly from R with the .Call interface.
 */
inline SEXP
tag (SEXP ptr, SEXP r_tokens)
{
  PyObject *tokens, *tagger, *taggedtokens, *funcname;
  SEXP r_taggedtokens;
  
  /* Convert to a python token list */
  tokens = r2p_token_list(r_tokens);

  /* Grab tagger from external pointer */
  tagger = (PyObject*) R_ExternalPtrAddr(ptr);
  if (tagger == NULL)
    error("Tagger address is invalid");
  
  /* Call the tagger's tag method */
  funcname = PyString_FromString("tag");
  taggedtokens = PyObject_CallMethodObjArgs(tagger, funcname, tokens);
  Py_DECREF(funcname); /* cleaning */
  Py_DECREF(tokens);            /* up */

  /* Convert back to R */
  r_taggedtokens = p2r_tagged_token_list2matrix(taggedtokens);
  Py_DECREF(taggedtokens); // done with taggedtokens

  /* Unprotect r_taggedtokens and return */
  UNPROTECT(1);
  return r_taggedtokens;
}

/* Takes a SEXP containing either an R external pointer to a backoff
 * tagger or R's NULL and returns either the PyObject* representing the
 * backoff tagger or Py_None.
 */
inline PyObject*
get_backoff (SEXP r_backoff)
{
  PyObject *backoff;
  
  if (r_backoff != R_NilValue)
    backoff = (PyObject*) R_ExternalPtrAddr(r_backoff);
  else
    backoff = Py_None;

  return backoff;
}

#endif /* __TAGGER_H__ */
