library(text2num)

# Read a file
path <- "inst/samples/en/anna.txt"
myd <- readDocument(path)

# A document has an id and a string of text
print(myd)
print(id(myd))
print(content(myd))


# Basic operations on documents

# Tokenize myd using the default tokenizer
myd.t <- tokenize(myd)

# Print the first five tokens in myd
print(myd.t[1:5])

# Remove case and stem using the default stemmer
myd.st <- stem(lowercase(myd.t))

# Print the first five tokens again
print(myd.st[1:5])

# These same operations work on plain character vectors, too
mystr <- "Going to the store for eggs."
mytoks <- stem(lowercase(mystr))
print(mytoks)

# The default tokenizer uses regular expressions. This can be changed.
# This example keeps "to the" as a single token
mytoks <- tokenize(mystr, RegexTokenizer("(to the)|[\\w]+"))
print(mytoks)

# This example matches gaps between tokens, instead of tokens
mytoks <- tokenize(mystr, RegexTokenizer("t", matchToken=FALSE))
print(mytoks)


# The default stemmer is Porter's English stemmer in the Rstem package, but
# this can be changed. Using Spanish gives a warning from the Rstem package.
mystr.sp <- "Yo tengo una guitarra"
stem(mystr.sp, SnowballStemmer("spanish"))  


# Basic token frequency statistics
print(myd.t)
print(unique(myd.t))
print(freqTable(myd.t))


# Can filter out tokens and apply functions to each token

fltr <- function(x) return(nchar(x) < 5)
myd.t.long <- filterTokens(myd.t, fltr)
print(unique(myd.t.long))

myFn <- function(x) paste(x, x, sep="")
myd.t.twice <- applyToTokens(myd.t, myFn)
print(myd.t.twice[1:5])

# Filter by regex
myd.t.noh <- filterTokensRegex(myd.t, "^[HhFf]", negate=TRUE)
print(myd.t.noh[1:5])


# Can do all of the above for corpora

# Read in new corpus
myc <- readCorpus("inst/samples/en")

print(myc)

# Normalize the corpus
myc.norm <- stem(lowercase(myc))

print(myc.norm)

# Can get some frequencies

mymat <- documentTokenMatrix(myc.norm)
print(mymat)
print(mymat["anna.txt", "the"])
print(mymat[,"the"])
mymat[,c("the","a","of")]
print(sum(mymat))


# Kevin's example of removing low frequency tokens in the corpus (will
# incorporate this into the library later):

# Get the total frequencies of each token in the corpus
totalFreqs <- apply(mymat, 2, sum)

# Make a filter function
fltr <- function(x) 2 < as.vector(totalFreqs[x])

# Filter the tokens
myc.filtered <- filterTokens(myc.norm, fltr)
print(documentTokenMatrix(myc.filtered))

# Unigram tagger example
orig <- c("The", "at", "Fulton", "np-tl", "Country", "nn-tl",
"Grand", "jj-tl", "Jury", "nn-tl", "said", "vbd", "Friday", "nr",
"an", "at", "investigation", "nn", "of", "in", "Atlanta's", "np$",
"recent", "jj", "primary", "nn", "election", "nn", "produced", "vbd",
"''", "''", "no", "at", "evidence", "nn", "''", "''", "that", "cs",
"any", "dti", "irregularities", "nns", "took", "vbd", "place", "nn",
".", ".")

test <- orig[seq(1,49,2)]

# Notice is missing the word any, for backoff check
train <- list(list("The", "AT"), list("Fulton", "NP-TL"),
list("Country", "NN-Tl"), list("Grand", "JJ-TL"), list("Jury",
"NN-TL"), list("said", "VBD"), list("Friday", "NR"), list("an", "AT"),
list("investigation", "NN"), list("of", "IN"), list("Atlanta's",
"NP$"), list("recent", "JJ"), list("primary", "NN"), list("election",
"NN"), list("produced", "VBD"), list("''", "''"), list("no", "AT"),
list("evidence", "NN"), list("''", "''"), list("that", "CS"),
list("irregularities", "NNS"), list("took",
"VBD"), list("place", "NN"), list(".", "."))

train <- list(train, train)

utagger<-UnigramTagger(train, RegexTagger())
tagged.sentence <- tag(test, utagger)
tagged.doc <- tag(myd, utagger)
tagged.corpus <- tag(myc, utagger)
